use std::ops::{Deref, DerefMut};

use crate::{align_to, error::PEParseError};

use super::pe::{MutablePE, ReadOnlyPE};
use crate::constants::*;
pub struct MemoryPE {
    len: usize,
    buf: *const u8, // 缓冲区的指针 指向内存中的数据
}

pub struct MemoryPEMut {
    len: usize,
    buf: *mut u8, // 缓冲区的指针 指向内存中的数据
}
// DerefMut<Target = [u8]> + AsMut<[u8]>
impl MutablePE for MemoryPEMut{}

impl DerefMut for MemoryPEMut{
    fn deref_mut(&mut self) -> &mut Self::Target {
        unsafe {
            std::slice::from_raw_parts_mut(self.buf, self.len)
        }
    }
}

impl AsMut<[u8]> for MemoryPEMut{
    fn as_mut(&mut self) -> &mut [u8] {
        unsafe {
            std::slice::from_raw_parts_mut(self.buf, self.len)
        }
    }
}

impl Deref for MemoryPEMut{
    type Target = [u8];
    fn deref(&self) -> &Self::Target {
        unsafe {
            std::slice::from_raw_parts(self.buf, self.len)
        }
    }
}
impl ReadOnlyPE for MemoryPEMut{}

impl AsRef<[u8]> for MemoryPEMut{
    fn as_ref(&self) -> &[u8] {
        unsafe {
            std::slice::from_raw_parts(self.buf, self.len)
        }
    }
}

impl ReadOnlyPE for MemoryPE{}

impl Deref for MemoryPE{
    type Target = [u8];

    fn deref(&self) -> &Self::Target {
        unsafe {
            std::slice::from_raw_parts(self.buf, self.len)
        }
    }
}


impl AsRef<[u8]> for MemoryPE{
    fn as_ref(&self) -> &[u8] {
        unsafe {
            std::slice::from_raw_parts(self.buf, self.len)
        }
    }
}

/// 需要实现一个从内存中解析PE的方法
pub fn parse_pe_from_memory(buf: *const u8) -> Result<MemoryPE, PEParseError> {
    // 1. 检查是否为PE文件
    // 取出头两个字节
    let dos_magic = unsafe { *buf as u16 };
    if dos_magic != 0x5A4D { // 小尾
        return Err(PEParseError::InvalidDOSMagic);
    }
    // 从PE偏移处读取两个字节
    let pe_offset = unsafe { *(buf.add(DOS_HEADER_PE_OFFSET) as *const u32) };
    // 读取PE头部的标志
    let nt_signature = unsafe { *(buf.add(pe_offset as usize) as *const u32) };
    if nt_signature != NT_SIGNATURE {
        return Err(PEParseError::InvalidNTSignature);
    }
    // 检查结束
    // 从NT偏移处 读取一个NT header结构出来
    // 需要读取节对齐和镜像大小
    let section_alignment = unsafe { *(buf.add(pe_offset as usize + SECTION_ALIGNMENT_OFFSET) as *const u32) };
    let image_size = unsafe { *(buf.add(pe_offset as usize + IMAGE_SIZE_OFFSET) as *const u32) };
    // 计算对齐后的内存大小
    let aligned_image_size = align_to!(image_size, section_alignment);
    Ok(MemoryPE {
        len: aligned_image_size as usize,
        buf,
    })
}

/// 还需要一个Mut的版本
pub fn parse_pe_from_memory_mut(buf: *mut u8) -> Result<MemoryPEMut, PEParseError> {
    // 1. 检查是否为PE文件
    // 取出头两个字节
    let dos_magic = unsafe { *buf as u16 };
    if dos_magic != 0x5A4D { // 小尾
        return Err(PEParseError::InvalidDOSMagic);
    }
    // 从PE偏移处读取两个字节
    let pe_offset = unsafe { *(buf.add(DOS_HEADER_PE_OFFSET) as *const u32) };
    // 读取PE头部的标志
    let nt_signature = unsafe { *(buf.add(pe_offset as usize) as *const u32) };
    if nt_signature != NT_SIGNATURE {
        return Err(PEParseError::InvalidNTSignature);
    }
    // 检查结束
    // 从NT偏移处 读取一个NT header结构出来
    // 需要读取节对齐和镜像大小
    let section_alignment = unsafe { *(buf.add(pe_offset as usize + SECTION_ALIGNMENT_OFFSET) as *const u32) };
    let image_size = unsafe { *(buf.add(pe_offset as usize + IMAGE_SIZE_OFFSET) as *const u32) };
    // 计算对齐后的内存大小
    let aligned_image_size = align_to!(image_size, section_alignment);
    Ok(MemoryPEMut {
        len: aligned_image_size as usize,
        buf,
    })
}